# -*- coding: utf-8 -*-
#------------------------------------------------------------
# API for RuYa IPTV
# Version 1.0.1
#------------------------------------------------------------
# Changes
# 1.0.1 Added quality selector for movies 
#------------------------------------------------------------

import os
import sys
import urlparse
import plugintools
import re

import urllib
from twisted.web.client import getPage

# ---------------------------------------------------------------------------------------------------------
#  users
# ---------------------------------------------------------------------------------------------------------
def get_base_url():
    return "http://"+plugintools.get_setting("server")+"/ruyaserver/"

def login(server,username,password):
    plugintools.log("ruyaiptv.api login")

    # Service call
    try:
        service_url = urlparse.urljoin(get_base_url(),"users/login.php")
        service_parameters = urllib.urlencode({'username':username,'password':password})
        body , response_headers = plugintools.read_body_and_headers( service_url , post=service_parameters )
    except:
        body = ""

    return body

def get_cover_backdrops(server):
    plugintools.log("ruyaiptv.api get_cover_backdrops")

    # Service call
    try:
        service_url = urlparse.urljoin(get_base_url(),"get_cover_backdrops.php")
        service_parameters = ""
        body , response_headers = plugintools.read_body_and_headers( service_url , post=service_parameters )
        plugintools.log("ruyaiptv.api get_cover_backdrops body="+body)
        cover_backdrops = body.split("|")
    except:
        body = ""
        cover_backdrops = []

    plugintools.log("ruyaiptv.api get_cover_backdrops cover_backdrops="+repr(cover_backdrops))

    return cover_backdrops

# ---------------------------------------------------------------------------------------------------------
#  movies
# ---------------------------------------------------------------------------------------------------------

def get_recent_movies(token,num_page="",per_page="",callback=None,action="",title=""):
    plugintools.log("ruyaiptv.api.get_recent_movies")

    # Service call
    service_url = urlparse.urljoin(get_base_url(),"movies/get_recent_movies.php")
    service_parameters = urllib.urlencode({'token':token,'num_page':num_page,'per_page':per_page})
    getPage(service_url,method='POST',postdata=service_parameters,headers={'Content-Type':'application/x-www-form-urlencoded'}).addCallback(get_recent_movies_callback,callback,action,title,num_page).addErrback(generic_errback)

def get_recent_movies_callback(data, callback=None, action="", title="",num_page=""):
    plugintools.loginfo("ruyaiptv.api.get_recent_movies_callback data="+str(data))

    # Response parsing
    plugintools.log("ruyaiptv.api.get_recent_movies_callback data="+repr(data))
    items = parse_movies(data)
    plugintools.log("ruyaiptv.api.get_recent_movies_callback items="+repr(items))

    # Adds new page
    if len(items)==13:
        next_page = str(int(num_page)+1)
        items.append({"title":"More...","thumbnail":"http://"+plugintools.get_setting("server")+"/ruyaserver/img/next.png","action":"movies_recent","extra":"","num_page":next_page,"per_page":"13", "plot":""})

    #back_item = {"title":"Previous","thumbnail":"/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/nothumb-movies.png","action":"back","extra":"","num_page":"","per_page":""}
    #items.reverse()
    #items.append(back_item)
    #items.reverse()

    callback(action,title,items)

def get_latest_releases(token,num_page="",per_page="",callback=None,action="",title=""):
    plugintools.log("ruyaiptv.api.get_latest_releases")

    # Service call
    service_url = urlparse.urljoin(get_base_url(),"movies/get_latest_releases.php")
    plugintools.log("ruyaiptv.api.get_latest_releases service_url="+service_url)
    service_parameters = urllib.urlencode({'token':token,'num_page':num_page,'per_page':per_page})
    plugintools.log("ruyaiptv.api.get_latest_releases service_parameters="+service_parameters)
    try:
        plugintools.log("ruyaiptv.api.get_latest_releases antes")
        getPage(service_url,method='POST',postdata=service_parameters,headers={'Content-Type':'application/x-www-form-urlencoded'}).addCallback(get_latest_releases_callback,callback,action,title,num_page).addErrback(generic_errback)
        plugintools.log("ruyaiptv.api.get_latest_releases despues")
    except:
        plugintools.log("ruyaiptv.api.get_latest_releases error")
        import traceback
        plugintools.loginfo("ruya.RuyaSectionMoviesScreen.action_ok "+traceback.format_exc())

def get_latest_releases_callback(data, callback=None, action="", title="",num_page=""):
    plugintools.loginfo("ruyaiptv.api.get_latest_releases_callback data="+str(data))

    # Response parsing
    plugintools.log("ruyaiptv.api.get_latest_releases_callback data="+repr(data))
    items = parse_movies(data)
    plugintools.log("ruyaiptv.api.get_latest_releases_callback items="+repr(items))

    # Adds new page
    if len(items)==13:
        next_page = str(int(num_page)+1)
        items.append({"title":"More...","thumbnail":"http://"+plugintools.get_setting("server")+"/ruyaserver/img/next.png","action":"latest_releases","extra":"","num_page":next_page,"per_page":"13", "plot":""})

    #back_item = {"title":"Previous","thumbnail":"/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/nothumb-movies.png","action":"back","extra":"","num_page":"","per_page":""}
    #items.reverse()
    #items.append(back_item)
    #items.reverse()

    callback(action,title,items)

def generic_errback(f):
    plugintools.loginfo("ruya.RuyaSectionMoviesScreen.getPageErrback post recibido con error")
    plugintools.loginfo("ruya.RuyaSectionMoviesScreen.getPageErrback "+repr(f))
    plugintools.loginfo("ruya.RuyaSectionMoviesScreen.getPageErrback "+repr(f.getTraceback()))

def movie_search(token,terms,num_page="",per_page="",callback=None,action="",title=""):
    plugintools.log("ruyaiptv.api.movie_search")

    # Service call
    service_url = urlparse.urljoin(get_base_url(),"movies/movie_search.php")
    service_parameters = urllib.urlencode({'token':token,'q':terms,'num_page':num_page,'per_page':per_page})
    body , response_headers = plugintools.read_body_and_headers( service_url , post=service_parameters )

    # Response parsing
    plugintools.log("ruyaiptv.api.movie_search body="+repr(body))
    items = parse_movies(body)
    plugintools.log("ruyaiptv.api.movie_search items="+repr(items))

    '''
    back_item = {"title":"Previous","thumbnail":"/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/nothumb-movies.png","action":"back","extra":"","num_page":"","per_page":""}
    #items.insert(back_item,0)
    items.reverse()
    items.append(back_item)
    items.reverse()
    '''

    return items

def get_movies_by_letter(token,letter,num_page="",per_page="",callback=None,action="",title=""):
    plugintools.log("ruyaiptv.api.get_movies_by_letter")

    # Service call
    service_url = urlparse.urljoin(get_base_url(),"movies/get_movies_by_letter.php")
    service_parameters = urllib.urlencode({'token':token,'letter':letter,'num_page':num_page,'per_page':per_page})
    getPage(service_url,method='POST',postdata=service_parameters,headers={'Content-Type':'application/x-www-form-urlencoded'}).addCallback(get_movies_by_letter_callback,callback,action,title,letter,num_page).addErrback(generic_errback)

def get_movies_by_letter_callback(data, callback=None, action="", title="", letter="",num_page=""):
    plugintools.loginfo("ruyaiptv.api.get_movies_by_letter_callback data="+str(data))

    # Response parsing
    items = parse_movies(data)
    plugintools.log("ruyaiptv.api.get_movies_by_letter_callback items="+repr(items))
    if len(items)==13:
        next_page = str(int(num_page)+1)
        items.append({"title":"More...","thumbnail":"http://"+plugintools.get_setting("server")+"/ruyaserver/img/next.png","action":"movies_by_letter","extra":letter,"num_page":next_page,"per_page":"13","plot":""})

    '''
    back_item = {"title":"Previous","thumbnail":"/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/nothumb-movies.png","action":"back","extra":"","num_page":"","per_page":""}
    #items.insert(back_item,0)
    items.reverse()
    items.append(back_item)
    items.reverse()
    '''
    callback(action,title,items)

def get_movie_genres(token,callback=None,action="",title=""):
    plugintools.log("ruyaiptv.api.get_movie_genres")

    # Service call
    service_url = urlparse.urljoin(get_base_url(),"movies/get_movie_genres.php")
    service_parameters = urllib.urlencode({'token':token})
    getPage(service_url,method='POST',postdata=service_parameters,headers={'Content-Type':'application/x-www-form-urlencoded'}).addCallback(get_movie_genres_callback,callback,action,title).addErrback(generic_errback)

def get_movie_genres_callback(data, callback=None, action="", title=""):
    plugintools.loginfo("ruyaiptv.api.get_movie_genres_callback data="+str(data))

    # Response parsing
    items = parse_movie_genres(data)
    plugintools.log("ruyaiptv.api.get_movie_genres_callback items="+repr(items))

    for item in items:
        item["action"]="movies_by_genre"

    callback(action,title,items)

def get_movies_by_genre(token,genre,num_page="",per_page="",callback=None,action="",title=""):
    plugintools.log("ruyaiptv.api.get_movies_by_genre")

    # Service call
    service_url = urlparse.urljoin(get_base_url(),"movies/get_movies_by_genre.php")
    service_parameters = urllib.urlencode({'token':token,'genre':genre,'num_page':num_page,'per_page':per_page})
    getPage(service_url,method='POST',postdata=service_parameters,headers={'Content-Type':'application/x-www-form-urlencoded'}).addCallback(get_movies_by_genre_callback,callback,action,title,genre,num_page).addErrback(generic_errback)

def get_movies_by_genre_callback(data, callback=None, action="", title="", genre="",num_page=""):
    plugintools.loginfo("ruyaiptv.api.get_movies_by_genre_callback data="+str(data))

    # Response parsing
    items = parse_movies(data)
    plugintools.log("ruyaiptv.api.get_movies_by_genre_callback items="+repr(items))
    if len(items)==13:
        next_page = str(int(num_page)+1)
        items.append({"title":"More...","thumbnail":"http://"+plugintools.get_setting("server")+"/ruyaserver/img/next.png","action":"movies_by_genre","extra":genre,"num_page":next_page,"per_page":"13","plot":""})

    '''
    back_item = {"title":"Previous","thumbnail":"/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/nothumb-movies.png","action":"back","extra":"","num_page":"","per_page":""}
    #items.insert(back_item,0)
    items.reverse()
    items.append(back_item)
    items.reverse()
    '''
    callback(action,title,items)

# ---------------------------------------------------------------------------------------------------------
#  tvshows
# ---------------------------------------------------------------------------------------------------------

def get_recent_episodes(token,num_page="",per_page="",callback=None,action="",title=""):
    plugintools.log("ruyaiptv.api.get_recent_episodes")

    # Service call
    service_url = urlparse.urljoin(get_base_url(),"tvshows/get_recent_episodes.php")
    service_parameters = urllib.urlencode({'token':token,'num_page':num_page,'per_page':per_page})
    getPage(service_url,method='POST',postdata=service_parameters,headers={'Content-Type':'application/x-www-form-urlencoded'}).addCallback(get_recent_episodes_callback,callback,action,title).addErrback(generic_errback)

def get_recent_episodes_callback(data, callback=None, action="", title=""):

    # Response parsing
    plugintools.log("ruyaiptv.api.get_recent_episodes body="+repr(data))
    items = parse_tvshow_episodes(data)
    plugintools.log("ruyaiptv.api.get_recent_episodes items="+repr(items))

    callback(action,title,items)

def get_updated_tvshows(token,num_page="",per_page="",callback=None,action="",title=""):
    plugintools.log("ruyaiptv.api.get_updated_tvshows")

    # Service call
    service_url = urlparse.urljoin(get_base_url(),"tvshows/get_updated_tvshows.php")
    service_parameters = urllib.urlencode({'token':token,'num_page':num_page,'per_page':per_page})
    getPage(service_url,method='POST',postdata=service_parameters,headers={'Content-Type':'application/x-www-form-urlencoded'}).addCallback(get_updated_tvshows_callback,callback,action,title,num_page).addErrback(generic_errback)

def get_updated_tvshows_callback(data, callback=None, action="", title="", num_page=""):

    # Response parsing
    plugintools.log("ruyaiptv.api.get_updated_tvshows body="+repr(data))
    items = parse_tvshows(data)
    plugintools.log("ruyaiptv.api.get_updated_tvshows items="+repr(items))

    for new_item in items:
        new_item["action"]="tvshows_seasons"

    '''
    if len(items)==7:
        next_page = str(int(num_page)+1)
        items.append({"title":"More...","http://"+plugintools.get_setting("server")+"/ruyaserver/img/next.png","action":"tvshows_updated","extra":"","num_page":next_page,"per_page":"25"})

    back_item = {"title":"Previous","thumbnail":"/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/nothumb-movies.png","action":"back","extra":"","num_page":"","per_page":""}
    #items.insert(back_item,0)
    items.reverse()
    items.append(back_item)
    items.reverse()
    '''

    callback(action,title,items)

def tvshow_search(token,terms,num_page="",per_page="",callback=None,action="",title=""):
    plugintools.log("ruyaiptv.api.tvshow_search")

    # Service call
    service_url = urlparse.urljoin(get_base_url(),"tvshows/tvshow_search.php")
    service_parameters = urllib.urlencode({'token':token,'q':terms,'num_page':num_page,'per_page':per_page})
    body , response_headers = plugintools.read_body_and_headers( service_url , post=service_parameters )

    # Response parsing
    plugintools.log("ruyaiptv.api.tvshow_search body="+repr(body))
    items = parse_tvshows(body)
    plugintools.log("ruyaiptv.api.tvshow_search items="+repr(items))

    for new_item in items:
        new_item["action"]="tvshows_seasons"

    '''
    back_item = {"title":"Previous","thumbnail":"/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/nothumb-movies.png","action":"back","extra":"","num_page":"","per_page":""}
    #items.insert(back_item,0)
    items.reverse()
    items.append(back_item)
    items.reverse()
    '''

    return items

def get_tvshows_by_letter(token,letter,num_page="",per_page="",callback=None,action="",title=""):
    plugintools.log("ruyaiptv.api.get_tvshows_by_letter")

    # Service call
    service_url = urlparse.urljoin(get_base_url(),"tvshows/get_tvshows_by_letter.php")
    service_parameters = urllib.urlencode({'token':token,'letter':letter,'num_page':num_page,'per_page':per_page})
    getPage(service_url,method='POST',postdata=service_parameters,headers={'Content-Type':'application/x-www-form-urlencoded'}).addCallback(get_tvshows_by_letter_callback,callback,action,title,letter,num_page).addErrback(generic_errback)

def get_tvshows_by_letter_callback(data, callback=None, action="", title="", letter="", num_page=""):

    # Response parsing
    plugintools.log("ruyaiptv.api.get_tvshows_by_letter body="+repr(data))
    items = parse_tvshows(data)
    plugintools.log("ruyaiptv.api.get_tvshows_by_letter items="+repr(items))

    for new_item in items:
        new_item["action"]="tvshows_seasons"
    '''
    if len(items)==7:
        next_page = str(int(num_page)+1)
        items.append({"title":"More...","thumbnail":"http://"+plugintools.get_setting("server")+"/ruyaserver/img/next.png","action":"tvshows_by_letter","extra":letter,"num_page":next_page,"per_page":"25"})

    back_item = {"title":"Previous","thumbnail":"/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/nothumb-movies.png","action":"back","extra":"","num_page":"","per_page":""}
    #items.insert(back_item,0)
    items.reverse()
    items.append(back_item)
    items.reverse()
    '''

    callback(action,title,items)

def get_tvshow_seasons(token,tvshow,num_page="",per_page="",callback=None,action="",title=""):
    plugintools.log("ruyaiptv.api.get_tvshow_seasons")

    # Service call
    service_url = urlparse.urljoin(get_base_url(),"tvshows/get_tvshow_seasons.php")
    service_parameters = urllib.urlencode({'token':token,'tvshow':tvshow,'num_page':num_page,'per_page':per_page})
    getPage(service_url,method='POST',postdata=service_parameters,headers={'Content-Type':'application/x-www-form-urlencoded'}).addCallback(get_tvshow_seasons_callback,callback,action,title,tvshow).addErrback(generic_errback)

def get_tvshow_seasons_callback(data, callback=None, action="", title="", tvshow=""):

    # Response parsing
    plugintools.log("ruyaiptv.api.get_tvshow_seasons body="+repr(data))
    items = parse_tvshow_seasons(data)
    plugintools.log("ruyaiptv.api.get_tvshow_seasons items="+repr(items))

    for new_item in items:
        new_item["action"]="tvshow_episodes"
        new_item["extra"]=tvshow

    callback(action,title,items)

def get_tvshow_episodes(token,tvshow,season,num_page="",per_page="",callback=None,action="",title=""):
    plugintools.log("ruyaiptv.api.get_tvshow_episodes")

    # Service call
    service_url = urlparse.urljoin(get_base_url(),"tvshows/get_tvshow_episodes.php")
    service_parameters = urllib.urlencode({'token':token,'tvshow':tvshow,'season':season,'num_page':num_page,'per_page':per_page})
    getPage(service_url,method='POST',postdata=service_parameters,headers={'Content-Type':'application/x-www-form-urlencoded'}).addCallback(get_tvshow_episodes_callback,callback,action,title,tvshow,season).addErrback(generic_errback)

def get_tvshow_episodes_callback(data, callback=None, action="", title="", tvshow="", season=""):

    # Response parsing
    plugintools.log("ruyaiptv.api.get_tvshow_episodes body="+repr(data))
    items = parse_tvshow_episodes(data)
    plugintools.log("ruyaiptv.api.get_tvshow_episodes items="+repr(items))

    callback(action,title,items)

def get_tvshow_genres(token,callback=None,action="",title=""):
    plugintools.log("ruyaiptv.api.get_tvshow_genres")

    # Service call
    service_url = urlparse.urljoin(get_base_url(),"tvshows/get_tvshow_genres.php")
    service_parameters = urllib.urlencode({'token':token})
    getPage(service_url,method='POST',postdata=service_parameters,headers={'Content-Type':'application/x-www-form-urlencoded'}).addCallback(get_tvshow_genres_callback,callback,action,title).addErrback(generic_errback)

def get_tvshow_genres_callback(data, callback=None, action="", title=""):

    # Response parsing
    plugintools.log("ruyaiptv.api.get_tvshow_genres body="+repr(data))
    items = parse_tvshow_genres(data)
    plugintools.log("ruyaiptv.api.get_tvshow_genres items="+repr(items))

    for new_item in items:
        new_item["action"]="tvshows_by_genre"

    callback(action,title,items)

def get_tvshows_by_genre(token,genre,num_page="",per_page="",callback=None,action="",title=""):
    plugintools.log("ruyaiptv.api.get_tvshows_by_genre")

    # Service call
    service_url = urlparse.urljoin(get_base_url(),"tvshows/get_tvshows_by_genre.php")
    service_parameters = urllib.urlencode({'token':token,'genre':genre,'num_page':num_page,'per_page':per_page})
    getPage(service_url,method='POST',postdata=service_parameters,headers={'Content-Type':'application/x-www-form-urlencoded'}).addCallback(get_tvshows_by_genre_callback,callback,action,title,genre,num_page).addErrback(generic_errback)

def get_tvshows_by_genre_callback(data, callback=None, action="", title="", genre="", num_page=""):

    # Response parsing
    plugintools.log("ruyaiptv.api.get_tvshows_by_genre body="+repr(data))
    items = parse_tvshows(data)
    plugintools.log("ruyaiptv.api.get_tvshows_by_genre items="+repr(items))

    for new_item in items:
        new_item["action"]="tvshows_seasons"

    '''
    if len(items)==25:
        next_page = str(int(num_page)+1)
        items.append({"title":"More...","thumbnail":"/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/nothumb-movies.png","action":"tvshows_by_genre","extra":genre,"num_page":next_page,"per_page":"25"})

    back_item = {"title":"Previous","thumbnail":"/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/nothumb-movies.png","action":"back","extra":"","num_page":"","per_page":""}
    #items.insert(back_item,0)
    items.reverse()
    items.append(back_item)
    items.reverse()
    '''

    callback(action,title,items)

# ---------------------------------------------------------------------------------------------------------
#  livetv
# ---------------------------------------------------------------------------------------------------------

def get_livetv_genres(token,callback=None,action="",title=""):
    plugintools.log("ruyaiptv.api.get_livetv_genres")

    # Service call
    service_url = urlparse.urljoin(get_base_url(),"livetv/get_livetv_genres.php")
    service_parameters = urllib.urlencode({'token':token})
    getPage(service_url,method='POST',postdata=service_parameters,headers={'Content-Type':'application/x-www-form-urlencoded'}).addCallback(get_livetv_genres_callback,callback,action,title).addErrback(generic_errback)

def get_livetv_genres_callback(data, callback=None, action="", title=""):

    # Response parsing
    plugintools.log("ruyaiptv.api.get_livetv_genres_callback body="+repr(data))
    items = parse_livetv_genres(data)
    plugintools.log("ruyaiptv.api.get_livetv_genres_callback items="+repr(items))

    filtered_items = []
    for item in items:
        item["action"]="livetv_channels_by_genre"

        if plugintools.get_setting("show_adult")=="false":
            plugintools.log("ruyaiptv.api.get_livetv_genres_callback dont show adult")
            if not is_adult_content(item["title"]):            
                plugintools.log("ruyaiptv.api.get_livetv_genres_callback "+item["title"]+" is not adult")
                filtered_items.append(item)
        else:
            plugintools.log("ruyaiptv.api.get_livetv_genres_callback show adult")
            filtered_items.append(item)

    callback(action,title,filtered_items)

def get_livetv_packages(token,callback=None,action="",title=""):
    plugintools.log("ruyaiptv.api.get_livetv_packages")

    # Service call
    service_url = urlparse.urljoin(get_base_url(),"livetv/get_livetv_packages.php")
    service_parameters = urllib.urlencode({'token':token})
    getPage(service_url,method='POST',postdata=service_parameters,headers={'Content-Type':'application/x-www-form-urlencoded'}).addCallback(get_livetv_packages_callback,callback,action,title).addErrback(generic_errback)

def get_livetv_packages_callback(data, callback=None, action="", title=""):

    # Response parsing
    plugintools.log("ruyaiptv.api.get_livetv_packages_callback body="+repr(data))
    items = parse_livetv_packages(data)
    plugintools.log("ruyaiptv.api.get_livetv_packages_callback items="+repr(items))

    filtered_items = []
    for item in items:
        item["action"]="livetv_channels_by_package"
        if plugintools.get_setting("show_adult")=="false":
            plugintools.log("ruyaiptv.api.get_livetv_packages_callback dont show adult")
            if not is_adult_content(item["title"]):            
                plugintools.log("ruyaiptv.api.get_livetv_packages_callback "+item["title"]+" is not adult")
                filtered_items.append(item)
        else:
            plugintools.log("ruyaiptv.api.get_livetv_packages_callback show adult")
            filtered_items.append(item)

    plugintools.log("ruyaiptv.api.get_livetv_packages_callback filtered_items="+repr(filtered_items))

    callback(action,title,filtered_items)

def get_all_livetv_channels(token,num_page="",per_page="",callback=None,action="",title=""):
    plugintools.log("ruyaiptv.api.get_all_livetv_channels")

    # Service call
    service_url = urlparse.urljoin(get_base_url(),"livetv/get_all_livetv_channels.php")
    service_parameters = urllib.urlencode({'token':token,'num_page':num_page,'per_page':per_page,'device':'e2'})
    getPage(service_url,method='POST',postdata=service_parameters,headers={'Content-Type':'application/x-www-form-urlencoded'}).addCallback(get_all_livetv_channels_callback,callback,action,title).addErrback(generic_errback)

def get_all_livetv_channels_callback(data, callback=None, action="", title=""):

    # Response parsing
    try:
        import gzip
        from StringIO import StringIO
        plugintools.log("get_all_livetv_channels_callback Gzip supported")
        buf = StringIO(data)
        f = gzip.GzipFile(fileobj=buf)
        data = f.read()
    except:
        _log("get_all_livetv_channels_callback Gzip not supported")

    plugintools.log("ruyaiptv.api.get_all_livetv_channels body="+repr(data))
    items = parse_livetv_channels(data)
    plugintools.log("ruyaiptv.api.get_all_livetv_channels items="+repr(items))

    callback(action,title,items)

def get_livetv_channels_by_genre(token,genre,num_page="",per_page="",callback=None,action="",title=""):
    plugintools.log("ruyaiptv.api.get_livetv_channels_by_genre")

    # Service call
    service_url = urlparse.urljoin(get_base_url(),"livetv/get_livetv_channels_by_genre.php")
    service_parameters = urllib.urlencode({'token':token,'genre':genre,'num_page':num_page,'per_page':per_page,'device':'e2'})
    getPage(service_url,method='POST',postdata=service_parameters,headers={'Content-Type':'application/x-www-form-urlencoded'}).addCallback(get_livetv_channels_by_genre_callback,callback,action,title,genre).addErrback(generic_errback)

def get_livetv_channels_by_genre_callback(data, callback=None, action="", title="", genre=""):

    # Response parsing
    plugintools.log("ruyaiptv.api.get_livetv_channels_by_genre body="+repr(data))
    items = parse_livetv_channels(data)
    plugintools.log("ruyaiptv.api.get_livetv_channels_by_genre items="+repr(items))

    callback(action,title,items)

def get_livetv_channels_by_package(token,package,num_page="",per_page="",callback=None,action="",title=""):
    plugintools.log("ruyaiptv.api.get_livetv_channels_by_package")

    # Service call
    service_url = urlparse.urljoin(get_base_url(),"livetv/get_livetv_channels_by_package.php")
    service_parameters = urllib.urlencode({'token':token,'package':package,'num_page':num_page,'per_page':per_page,'device':'e2'})
    getPage(service_url,method='POST',postdata=service_parameters,headers={'Content-Type':'application/x-www-form-urlencoded'}).addCallback(get_livetv_channels_by_package_callback,callback,action,title,package).addErrback(generic_errback)

def get_livetv_channels_by_package_callback(data, callback=None, action="", title="", package=""):

    # Response parsing
    plugintools.log("ruyaiptv.api.get_livetv_channels_by_package body="+repr(data))
    items = parse_livetv_channels(data)
    plugintools.log("ruyaiptv.api.get_livetv_channels_by_package items="+repr(items))

    callback(action,title,items)

# ---------------------------------------------------------------------------------------------------------
#  parsers
# ---------------------------------------------------------------------------------------------------------

def parse_movies(body):

    patron = "<movie>(.*?)</movie>"
    movies = plugintools.find_multiple_matches(body,patron)

    items = []
    for movie in movies:
        item = {}
        item["title"] = plugintools.find_single_match(movie,"<title>([^<]+)</title>")

        item["url"] = ""
        low_quality_url = plugintools.find_single_match(movie,"<low_quality_url>([^<]+)</low_quality_url>")
        if low_quality_url != "":
            item["url"] = item["url"] + low_quality_url

        high_quality_url = plugintools.find_single_match(movie,"<high_quality_url>([^<]+)</high_quality_url>")
        if high_quality_url != "":
            if item["url"]!="":
                item["url"] = item["url"] + "|"
            item["url"] = item["url"] + high_quality_url

        # There can be one quality (low) or two qualities (low|high). If there is none of them, add the original quality as the only one
        if item["url"]=="":
            item["url"] = plugintools.find_single_match(movie,"<url>([^<]+)</url>")

        item["thumbnail"] = plugintools.find_single_match(movie,"<thumbnail>([^<]+)</thumbnail>")
        item["fanart"] = plugintools.find_single_match(movie,"<fanart>([^<]+)</fanart>")
        item["plot"] = plugintools.find_single_match(movie,"<plot>([^<]+)</plot>")

        if item["title"]!="" and item["url"]!="":
            items.append(item)

    if len(items)==0:
        item = {}
        item["title"] = "(Empty list)"
        item["url"] = ""
        item["thumbnail"] = ""
        item["plot"] = ""
        item["action"] = ""
        items.append(item)

    return items

def parse_movie_genres(body):

    patron = "<genre>([^<]+)</genre>"
    genres = plugintools.find_multiple_matches(body,patron)

    items = []
    for genre in genres:
        item = {}
        item["title"] = genre
        items.append(item)

    if len(items)==0:
        item = {}
        item["title"] = "(Empty list)"
        item["url"] = ""
        item["thumbnail"] = ""
        item["plot"] = ""
        item["action"] = ""
        items.append(item)

    return items

def parse_tvshows(body):
    plugintools.log("ruyaiptv.api.parse_tvshows")

    patron = "<tvshow>(.*?)</tvshow>"
    tvshows = plugintools.find_multiple_matches(body,patron)

    items = []
    for tvshow in tvshows:
        item = {}
        item["title"] = plugintools.find_single_match(tvshow,"<title>([^<]+)</title>")
        item["thumbnail"] = plugintools.find_single_match(tvshow,"<thumbnail>([^<]+)</thumbnail>")
        item["fanart"] = plugintools.find_single_match(tvshow,"<fanart>([^<]+)</fanart>")
        item["plot"] = plugintools.find_single_match(tvshow,"<plot>([^<]+)</plot>")

        if item["title"]!="":
            items.append(item)

    if len(items)==0:
        item = {}
        item["title"] = "(Empty list)"
        item["url"] = ""
        item["thumbnail"] = ""
        item["plot"] = ""
        item["action"] = ""
        items.append(item)

    return items

def parse_tvshow_seasons(body):
    plugintools.log("ruyaiptv.api.parse_tvshow_episodes")

    patron = "<season>(.*?)</season>"
    episodes = plugintools.find_multiple_matches(body,patron)

    items = []
    for episode in episodes:
        item = {}
        item["title"] = plugintools.find_single_match(episode,"<title>([^<]+)</title>")
        item["thumbnail"] = plugintools.find_single_match(episode,"<thumbnail>([^<]+)</thumbnail>")
        item["fanart"] = plugintools.find_single_match(episode,"<fanart>([^<]+)</fanart>")
        item["plot"] = ""

        if item["title"]!="":
            items.append(item)

    if len(items)==0:
        item = {}
        item["title"] = "(Empty list)"
        item["url"] = ""
        item["thumbnail"] = ""
        item["plot"] = ""
        item["action"] = ""
        items.append(item)

    return items

def parse_tvshow_episodes(body):
    plugintools.log("ruyaiptv.api.parse_tvshow_episodes")

    patron = "<episode>(.*?)</episode>"
    episodes = plugintools.find_multiple_matches(body,patron)

    items = []
    for episode in episodes:
        item = {}
        item["title"] = plugintools.find_single_match(episode,"<title>([^<]+)</title>")
        item["thumbnail"] = plugintools.find_single_match(episode,"<thumbnail>([^<]+)</thumbnail>")
        item["fanart"] = plugintools.find_single_match(episode,"<fanart>([^<]+)</fanart>")
        item["plot"] = plugintools.find_single_match(episode,"<plot>([^<]+)</plot>")

        item["url"] = ""
        low_quality_url = plugintools.find_single_match(episode,"<low_quality_url>([^<]+)</low_quality_url>")
        if low_quality_url != "":
            item["url"] = item["url"] + low_quality_url

        high_quality_url = plugintools.find_single_match(episode,"<high_quality_url>([^<]+)</high_quality_url>")
        if high_quality_url != "":
            if item["url"]!="":
                item["url"] = item["url"] + "|"
            item["url"] = item["url"] + high_quality_url

        # There can be one quality (low) or two qualities (low|high). If there is none of them, add the original quality as the only one
        if item["url"]=="":
            item["url"] = plugintools.find_single_match(episode,"<url>([^<]+)</url>")

        if item["title"]!="":
            items.append(item)

    if len(items)==0:
        item = {}
        item["title"] = "(Empty list)"
        item["url"] = ""
        item["thumbnail"] = ""
        item["plot"] = ""
        item["action"] = ""
        items.append(item)

    return items

def parse_tvshow_genres(body):

    patron = "<genre>([^<]+)</genre>"
    genres = plugintools.find_multiple_matches(body,patron)

    items = []
    for genre in genres:
        item = {}
        item["title"] = genre
        items.append(item)

    if len(items)==0:
        item = {}
        item["title"] = "(Empty list)"
        item["url"] = ""
        item["thumbnail"] = ""
        item["plot"] = ""
        item["action"] = ""
        items.append(item)

    return items

def parse_livetv_genres(body):

    patron = "<genre>([^<]+)</genre>"
    genres = plugintools.find_multiple_matches(body,patron)

    items = []
    for genre in genres:
        item = {}
        item["title"] = genre
        items.append(item)

    if len(items)==0:
        item = {}
        item["title"] = "(Empty list)"
        item["url"] = ""
        item["thumbnail"] = ""
        item["plot"] = ""
        item["action"] = ""
        items.append(item)

    return items

def parse_livetv_packages(body):

    patron = "<package>([^<]+)</package>"
    packages = plugintools.find_multiple_matches(body,patron)

    items = []
    for package in packages:
        item = {}
        item["title"] = package
        items.append(item)

    if len(items)==0:
        item = {}
        item["title"] = "(Empty list)"
        item["url"] = ""
        item["thumbnail"] = ""
        item["plot"] = ""
        item["action"] = ""
        items.append(item)

    return items

def parse_livetv_channels(body):

    patron = "<channel>(.*?)</channel>"
    channels = plugintools.find_multiple_matches(body,patron)

    items = []
    for channel in channels:

        adult = plugintools.find_single_match(channel,"<adult>([^<]+)</adult>")
        title = plugintools.find_single_match(channel,"<name>([^<]+)</name>")

        if is_adult_content(title):
            adult="true"

        if plugintools.get_setting("show_adult")=="true" or (plugintools.get_setting("show_adult")=="false" and adult!="true"):
            item = {}
            item["title"] = title

            item["url"] = plugintools.find_single_match(channel,"<stream_url>([^<]+)</stream_url>")
            if item["url"].startswith("movie://"):
                item["url"] = item["url"].replace("movie://","")
            item["thumbnail"] = plugintools.find_single_match(channel,"<piconname>([^<]+)</piconname>")
            item["plot"] = plugintools.find_single_match(channel,"<plot>([^<]+)</plot>")
            item["fanart"] = adult
            low_quality_url = plugintools.find_single_match(channel,"<stream_url_sd>([^<]+)</stream_url_sd>")
            high_quality_url = plugintools.find_single_match(channel,"<stream_url_hd>([^<]+)</stream_url_hd>")

            if low_quality_url != "" and high_quality_url != "":
                item["url"] = low_quality_url + "|" + high_quality_url

            if item["title"]!="" and item["url"]!="":

                try:
                    if item["title"].strip().startswith("---"):
                        item["url"]=""
                    item["title"] = format_title(item["title"])
                except:
                    plugintools.log("ruyaiptv.api.parse_livetv_channels Error replacing title")

                items.append(item)

    if len(items)==0:
        item = {}
        item["title"] = "(Empty list)"
        item["url"] = ""
        item["thumbnail"] = ""
        item["plot"] = ""
        item["action"] = ""
        items.append(item)

    return items

def format_title(title):
    if title.strip().startswith("---"):
        new_title = re.compile("[\-]+",re.DOTALL).sub("",title).strip()
    else:
        new_title = "  "+title

    return new_title

def get_livetv_url(token,url):
    plugintools.log("ruyaiptv.api.get_livetv_url")

    import base64
    url = base64.b64decode(url)

    # Service call
    body , response_headers = plugintools.read_body_and_headers( url )

    mediaurl = base64.b64decode(body)

    return mediaurl

def is_adult_content(title):
    return title.lower()=="adult" or title.lower()=="xxx" or title.replace("-","").strip().lower()=="adult"

# ---------------------------------------------------------------------------------------------------------
#  fixme: old test function
# ---------------------------------------------------------------------------------------------------------

def read_server_response(url):

    if url=="movies":
        f = open( os.path.join( plugintools.get_runtime_path(),"api-movies.xml") ,"r")
        body = f.read()
        f.close()
    elif url=="tvshows":
        f = open( os.path.join( plugintools.get_runtime_path(),"api-tvshows.xml") ,"r")
        body = f.read()
        f.close()
    elif url=="livetv":
        f = open( os.path.join( plugintools.get_runtime_path(),"api-livetv.xml") ,"r")
        body = f.read()
        f.close()

    return body