# -*- coding: utf-8 -*-
#---------------------------------------------------------------------------
# Plugin Tools Enigma 2 - v1.0.0
#---------------------------------------------------------------------------
# License: GPL (http://www.gnu.org/licenses/gpl-3.0.html)
# Based on code from Project Valerie and Multimediathek plugins
# Author: 
# Jesús
# tvalacarta@gmail.com
# http://www.mimediacenter.info/plugintools
#---------------------------------------------------------------------------
# TODO:
# Support for gzip response encoding
#---------------------------------------------------------------------------

import urllib
import urllib2
import re
import sys
import os
import time
import socket
from StringIO import StringIO

module_log_enabled = True
http_debug_log_enabled = False
overrides = dict()

# Write something on enigma log (Coloring logs taken from ProjectValerie ;)
def logerror(message):
    print '\033[0;31m' + "E" + "  " + str(message) + '\033[1;m'

def loginfo(message):
    print '\033[0;34m' + "I" + "  " + str(message) + '\033[1;m'

def logdebug(message):
    print '\033[0;36m' + "D" + "  " + str(message) + '\033[1;m'

def log(message):
    loginfo(message)

# Write this module messages on XBMC log
def _log(message):
    if module_log_enabled:
        logdebug("plugintools."+message)

# Fetch text content from an URL
def read(url):
    _log("read "+url)

    f = urllib2.urlopen(url)
    data = f.read()
    f.close()
    
    return data

def read_body_and_headers(url, post=None, headers=[], follow_redirects=False, timeout=None):
    _log("read_body_and_headers "+url)

    if post is not None:
        _log("read_body_and_headers post="+post)

    if len(headers)==0:
        headers.append(["User-Agent","Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:18.0) Gecko/20100101 Firefox/18.0"])

    # Start cookie lib
    ficherocookies = os.path.join( get_data_path(), 'cookies.dat' )
    _log("read_body_and_headers cookies_file="+ficherocookies)

    cj = None
    ClientCookie = None
    cookielib = None

    # Let's see if cookielib is available
    try:
        _log("read_body_and_headers importing cookielib")
        import cookielib
    except ImportError:
        _log("read_body_and_headers cookielib no disponible")
        # If importing cookielib fails
        # let's try ClientCookie
        try:
            _log("read_body_and_headers importing ClientCookie")
            import ClientCookie
        except ImportError:
            _log("read_body_and_headers ClientCookie not available")
            # ClientCookie isn't available either
            urlopen = urllib2.urlopen
            Request = urllib2.Request
        else:
            _log("read_body_and_headers ClientCookie available")
            # imported ClientCookie
            urlopen = ClientCookie.urlopen
            Request = ClientCookie.Request
            cj = ClientCookie.MozillaCookieJar()

    else:
        _log("read_body_and_headers cookielib available")
        # importing cookielib worked
        urlopen = urllib2.urlopen
        Request = urllib2.Request
        cj = cookielib.MozillaCookieJar()
        # This is a subclass of FileCookieJar
        # that has useful load and save methods

    if cj is not None:
    # we successfully imported
    # one of the two cookie handling modules
        _log("read_body_and_headers Cookies enabled")

        if os.path.isfile(ficherocookies):
            _log("read_body_and_headers Reading cookie file")
            # if we have a cookie file already saved
            # then load the cookies into the Cookie Jar
            try:
                cj.load(ficherocookies)
            except:
                _log("read_body_and_headers Wrong cookie file, deleting...")
                os.remove(ficherocookies)

        # Now we need to get our Cookie Jar
        # installed in the opener;
        # for fetching URLs
        if cookielib is not None:
            _log("read_body_and_headers opener using urllib2 (cookielib)")
            # if we use cookielib
            # then we get the HTTPCookieProcessor
            # and install the opener in urllib2
            if not follow_redirects:
                opener = urllib2.build_opener(urllib2.HTTPHandler(debuglevel=http_debug_log_enabled),urllib2.HTTPCookieProcessor(cj),NoRedirectHandler())
            else:
                opener = urllib2.build_opener(urllib2.HTTPHandler(debuglevel=http_debug_log_enabled),urllib2.HTTPCookieProcessor(cj))
            urllib2.install_opener(opener)

        else:
            _log("read_body_and_headers opener using ClientCookie")
            # if we use ClientCookie
            # then we get the HTTPCookieProcessor
            # and install the opener in ClientCookie
            opener = ClientCookie.build_opener(ClientCookie.HTTPCookieProcessor(cj))
            ClientCookie.install_opener(opener)

    # -------------------------------------------------
    # Cookies instaladas, lanza la petición
    # -------------------------------------------------

    # Contador
    inicio = time.clock()

    # Diccionario para las cabeceras
    txheaders = {}

    # Construye el request
    if post is None:
        _log("read_body_and_headers GET request")
    else:
        _log("read_body_and_headers POST request")
    
    # Añade las cabeceras
    _log("read_body_and_headers ---------------------------")
    for header in headers:
        _log("read_body_and_headers header %s=%s" % (str(header[0]),str(header[1])) )
        txheaders[header[0]]=header[1]
    _log("read_body_and_headers ---------------------------")

    req = Request(url, post, txheaders)
    if timeout is None:
        handle=urlopen(req)
    else:        
        #Disponible en python 2.6 en adelante --> handle = urlopen(req, timeout=timeout)
        #Para todas las versiones:
        try:
            import socket
            deftimeout = socket.getdefaulttimeout()
            socket.setdefaulttimeout(timeout)
            handle=urlopen(req)            
            socket.setdefaulttimeout(deftimeout)
        except:
            import sys
            for line in sys.exc_info():
                _log( "%s" % line )
    
    # Actualiza el almacén de cookies
    cj.save(ficherocookies)

    # Lee los datos y cierra
    data=handle.read()
    try:
        import gzip
        _log("read_body_and_headers Gzip supported")
        if handle.info().get('Content-Encoding') == 'gzip':
            _log("read_body_and_headers Gzip detected")
            buf = StringIO(data)
            f = gzip.GzipFile(fileobj=buf)
            data = f.read()
    except:
        _log("read_body_and_headers Gzip not supported")

    info = handle.info()
    _log("read_body_and_headers Response")

    returnheaders=[]
    _log("read_body_and_headers ---------------------------")
    for header in info:
        _log("read_body_and_headers "+header+"="+info[header])
        returnheaders.append([header,info[header]])
    handle.close()
    _log("read_body_and_headers ---------------------------")

    '''
    # Lanza la petición
    try:
        response = urllib2.urlopen(req)
    # Si falla la repite sustituyendo caracteres especiales
    except:
        req = urllib2.Request(url.replace(" ","%20"))
    
        # Añade las cabeceras
        for header in headers:
            req.add_header(header[0],header[1])

        response = urllib2.urlopen(req)
    '''
    
    # Tiempo transcurrido
    fin = time.clock()
    _log("read_body_and_headers Downloaded in %d seconds " % (fin-inicio+1))
    _log("read_body_and_headers body="+data)

    return data,returnheaders

class NoRedirectHandler(urllib2.HTTPRedirectHandler):
    def http_error_302(self, req, fp, code, msg, headers):
        infourl = urllib.addinfourl(fp, headers, req.get_full_url())
        infourl.status = code
        infourl.code = code
        return infourl
    http_error_300 = http_error_302
    http_error_301 = http_error_302
    http_error_303 = http_error_302
    http_error_307 = http_error_302

# Parse string and extracts multiple matches using regular expressions
def find_multiple_matches(text,pattern):
    _log("find_multiple_matches pattern="+pattern)
    
    matches = re.findall(pattern,text,re.DOTALL)

    return matches

# Parse string and extracts first match as a string
def find_single_match(text,pattern):
    _log("find_single_match pattern="+pattern)

    result = ""
    try:    
        matches = re.findall(pattern,text, flags=re.DOTALL)
        result = matches[0]
    except:
        result = ""

    return result

def get_temp_path():
    _log("get_temp_path")

    return "/tmp"

def get_runtime_path():
    _log("get_runtime_path")

    dev = os.path.abspath( os.path.dirname(__file__) )

    _log("get_runtime_path ->'"+str(dev)+"'")

    return dev

def get_data_path():
    _log("get_data_path")

    dev = "/media/hdd"

    _log("get_data_path ->'"+str(dev)+"'")

    return dev

def get_setting(name):
    _log("get_setting name='"+name+"'")

    if name in overrides:
        dev = overrides[name]
    else:
        dev = ""

    _log("get_setting ->'"+str(dev)+"'")

    return dev

def set_setting(name,value):
    _log("set_setting name='"+name+"','"+value+"'")

    overrides[name]=value

def open_settings_dialog():
    _log("open_settings_dialog")

    __settings__.openSettings()

def get_localized_string(code):
    _log("get_localized_string code="+str(code))

    dev = __language__(code)

    try:
        dev = dev.encode("utf-8")
    except:
        pass

    _log("get_localized_string ->'"+dev+"'")

    return dev

def keyboard_input(default_text="", title="", hidden=False):
    _log("keyboard_input default_text='"+default_text+"'")

    keyboard = xbmc.Keyboard(default_text,title,hidden)
    keyboard.doModal()
    
    if (keyboard.isConfirmed()):
        tecleado = keyboard.getText()
    else:
        tecleado = ""

    _log("keyboard_input ->'"+tecleado+"'")

    return tecleado

def message(text1, text2="", text3=""):
    if text3=="":
        xbmcgui.Dialog().ok( text1 , text2 )
    elif text2=="":
        xbmcgui.Dialog().ok( "" , text1 )
    else:
        xbmcgui.Dialog().ok( text1 , text2 , text3 )

def message_options(text1, text2="", text3=""):
    if text3=="":
        xbmcgui.Dialog().ok( text1 , text2 )
    elif text2=="":
        xbmcgui.Dialog().ok( "" , text1 )
    else:
        xbmcgui.Dialog().ok( text1 , text2 , text3 )


def get_md5(cadena):
    try:
        import hashlib
        devuelve = hashlib.md5(cadena).hexdigest()
    except:
        import md5
        import binascii
        devuelve = binascii.hexlify(md5.new(url).digest())
    
    return devuelve

def get_thumbnail_cache_folder( thumbnail_url ):

    if get_setting("cache_enabled")=="true":
        cache_path = os.path.join( get_setting("cache_path") , "ruya_cache" )
        if os.path.exists(cache_path):

            md5_thumbnail = get_md5( thumbnail_url )
            first_level = md5_thumbnail[0]
            second_level = md5_thumbnail[1]

            thumbnail_path = os.path.join( cache_path , first_level , second_level )

            os.system("mkdir -p "+thumbnail_path)

            return os.path.join( thumbnail_path , md5_thumbnail )
        else:
            return ""
    else:
        return ""

def rescale_skin(skin_data,skin_width,skin_height):
    _log("rescale_skin")

    from enigma import getDesktop
    desktop_width = getDesktop(0).size().width()
    desktop_height = getDesktop(0).size().height()
    _log("rescale_skin desktop width="+str(desktop_width)+", height="+str(desktop_height))

    if desktop_width==skin_width and desktop_height==skin_height:
        _log("rescale_skin No rescale")

    else:
        width_factor = float( float(desktop_width) / float(skin_width) )
        height_factor = float( float(desktop_height) / float(skin_height) )
        _log("rescale_skin width_factor="+repr(width_factor)+" height_factor="+repr(height_factor))

        matches = find_multiple_matches(skin_data,'position\="(\d+),(\d+)"')
        for element_width,element_height in matches:
            _log("rescale_skin element_width="+repr(element_width)+" element_height="+repr(element_height))
            new_width = round( int(element_width) * width_factor , 0 )
            new_height = round( int(element_height) * height_factor , 0 )
            _log("rescale_skin new_width="+repr(new_width)+" new_height="+repr(new_height))

            old_value = 'position="%s,%s"' % (element_width,element_height)
            new_value = 'position="%d,%d"' % (new_width,new_height)
            _log("rescale_skin from "+str(old_value)+" to "+str(new_value))

            skin_data = skin_data.replace(old_value,new_value)

        matches = find_multiple_matches(skin_data,'size\="(\d+),(\d+)"')
        for element_width,element_height in matches:
            _log("rescale_skin element_width="+repr(element_width)+" element_height="+repr(element_height))
            new_width = round( int(element_width) * width_factor , 0)
            new_height = round( int(element_height) * height_factor , 0)
            _log("rescale_skin new_width="+repr(new_width)+" new_height="+repr(new_height))

            old_value = 'size="%s,%s"' % (element_width,element_height)
            new_value = 'size="%d,%d"' % (new_width,new_height)
            _log("rescale_skin from "+str(old_value)+" to "+str(new_value))

            skin_data = skin_data.replace(old_value,new_value)

        matches = find_multiple_matches(skin_data,'font\="Bariol;(\d+)"')
        for font_size in matches:
            _log("rescale_skin font_size="+repr(font_size))
            new_size = round( int(font_size) * width_factor , 0)
            _log("rescale_skin new_size="+repr(new_size))

            old_value = 'font="Bariol;%s"' % (font_size)
            new_value = 'font="Bariol;%d"' % (new_size)
            _log("rescale_skin from "+str(old_value)+" to "+str(new_value))

            skin_data = skin_data.replace(old_value,new_value)

    return skin_data