# -*- coding: utf-8 -*-
#------------------------------------------------------------
# Enigma2 Plugin for RuYa IPTV
# Version 1.0.0
#------------------------------------------------------------
import os

from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Screens.InputBox import InputBox

from Components.Label import Label,MultiColorLabel
from Components.Input import Input
from Components.ActionMap import ActionMap
from Components.MenuList import MenuList
from Components.Pixmap import Pixmap, MovingPixmap
from Components.AVSwitch import AVSwitch

from enigma import ePicLoad
from enigma import eTimer
from Tools.LoadPixmap import LoadPixmap

import plugintools
import api

# For image loading
from twisted.web.client import downloadPage

from screen_custom_player import CustomPlayer

class RuyaBrowseMoviesScreen(Screen):
    infile = open("/usr/lib/enigma2/python/Plugins/Extensions/ruya/skin/browse-movies-list.xml","r")
    skin = infile.read()
    infile.close()
    skin = plugintools.rescale_skin(skin,1280,720)

    def __init__(self, session, itemlist = [], screen_title=""):
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.__init__")

        self.session = session
        self.itemlist = itemlist
        self.screen_title = screen_title
        Screen.__init__(self, session)

        self.NUM_ITEMS_ON_SCREEN=14

        # Setup widgets
        for index in range(0,self.NUM_ITEMS_ON_SCREEN):

            if index < len(itemlist):
                self["title_option"+str(index)] = MultiColorLabel(itemlist[index]["title"])
            else:
                self["title_option"+str(index)] = MultiColorLabel(" ")
                self["title_option"+str(index)].hide()

            self["focus"+str(index)] = Pixmap()
            self["focus"+str(index)].hide()

        self["screen_title"] = Label(screen_title)
        self["scrollbar_section"] = Label(" ")
        self["scrollbar_pointer"] = Label(" ")

        '''
        if len(self.itemlist)>self.NUM_ITEMS_ON_SCREEN:
            plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.__init__ scrollbar visible")
            self["scrollbar_section"].show()
            self["scrollbar_pointer"].show()
        else:
            plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.__init__ scrollbar hidden")
            self["scrollbar_section"].hide()
            self["scrollbar_pointer"].hide()
        '''
        self["scrollbar_section"].hide()
        self["scrollbar_pointer"].hide()

        self["background"] = Label(" ")
        self["thumbnail"] = Pixmap()

        if len(itemlist)>0:
            self["title"] = Label(self.itemlist[0]["title"])
            self["plot"] = Label(self.itemlist[0]["plot"])

        self["quality_selector_background"] = Pixmap()
        self["quality_selector_sd_focus"] = Pixmap()
        self["quality_selector_hd_focus"] = Pixmap()
        self["quality_selector_sd_text"] = MultiColorLabel("Watch in SD")
        self["quality_selector_hd_text"] = MultiColorLabel("Watch in HD")

        self["loading_background"]=Pixmap()
        self["loading_title"]=Label("Loading data...")
        
        self["loading_background"].hide()
        self["loading_title"].hide()

        # Setup keys
        self["myActionMap"] = ActionMap(["WizardActions"],
        {
            "ok": self.action_ok,
            "back": self.action_back,
            "up": self.action_up,
            "down": self.action_down,
            "right": self.action_right,
            "left": self.action_left
        }, -1)

        self.active = 0;
        self.first_item_on_screen = 0

        if len(self.itemlist)>self.NUM_ITEMS_ON_SCREEN-1:
            self.MAX_ITEM=self.NUM_ITEMS_ON_SCREEN-1
        else:
            self.MAX_ITEM=len(self.itemlist)-1

        # For image loading
        if len(self.itemlist)>0:
            self.thumbnail = self.itemlist[0]["thumbnail"]
            self.Scale = AVSwitch().getFramebufferScale()
            self.PicLoad = ePicLoad()
            self.PicLoad.PictureData.get().append(self.DecodePicture)
            self.onLayoutFinish.append(self.load_thumbnail)

        self.dialog_opened = False
        self.dialog_selected = "sd"
        self.hide_dialog()
        self.onLayoutFinish.append(self.move_focus)        

        self.Timer = eTimer()
        self.Timer.callback.append(self.show_loader)
        self.loader_frame_number=1
        self.thumbwidth = -1
        self.thumbheight = -1

    def show_loader(self):
        plugintools.loginfo("ruya.RuyaMainScreen.show_loader")

        self["loading_background"].show()
        self["loading_title"].show()
        pixmap = LoadPixmap("/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/aeon/loader-background-"+str(self.loader_frame_number)+".png")
        self["loading_background"].instance.setPixmap(pixmap)

        self.loader_frame_number = self.loader_frame_number + 1
        if self.loader_frame_number>8:
            self.loader_frame_number = 1

        self.Timer.start(250, False)

    def hide_loader(self):
        plugintools.loginfo("ruya.RuyaMainScreen.hide_loader")

        self.Timer.stop()
        self["loading_background"].hide()
        self["loading_title"].hide()

    def load_thumbnail(self):

        thumbnail = self.thumbnail
        thumbnail_cache = plugintools.get_thumbnail_cache_folder( thumbnail )

        if thumbnail_cache=="":

            plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.loadNextThumbnail Downloading "+thumbnail+"...")
            downloadPage(thumbnail, "/tmp/thumb").addCallback(self.thumbnail_loaded)

        else:

            if os.path.exists(thumbnail_cache):
                plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.loadNextThumbnail Cached "+thumbnail+"...")
                self.thumbnail_loaded(None)
            else:
                plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.loadNextThumbnail Downloading "+thumbnail+"...")
                downloadPage(thumbnail, thumbnail_cache ).addCallback(self.thumbnail_loaded)

    def thumbnail_loaded(self, picData):
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.thumbnail_loaded")

        try:
            self.PicLoad.setPara([ self["thumbnail"].instance.size().width() , self["thumbnail"].instance.size().height() , self.Scale[0] , self.Scale[1] , 0 , 1 , "#ff000000"])
            plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.thumbnail_loaded StartDecode")

            thumbnail = self.thumbnail
            thumbnail_cache = plugintools.get_thumbnail_cache_folder( thumbnail )
            if thumbnail_cache=="":
                thumbnail_cache="/tmp/thumb"

            self.PicLoad.startDecode(thumbnail_cache)
            plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.thumbnail_loaded End")
        except:
            import traceback
            plugintools.loginfo(traceback.format_exc())

    def DecodePicture(self, PicInfo = ""):
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.DecodePicture")
        ptr = self.PicLoad.getData()
        if ptr != None:
            self["thumbnail"].instance.setPixmap(ptr)

    def action_ok(self):
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.action_ok")
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.action_ok active=["+str(self.active)+"]")
        item = self.itemlist[ self.first_item_on_screen + self.active ]

        if self.dialog_opened:
            plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.action_ok url=["+item["url"]+"]")
            if self.dialog_selected == "sd":
                url = item["url"].split("|")[0]
            else:
                url = item["url"].split("|")[1]

            plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.action_ok url=["+url+"]")
            self.session.open(CustomPlayer, [{"title":item["title"],"url":url,"thumbnail":item["thumbnail"],"plot":item["plot"]}] , 0 )

        elif not "action" in item:
            plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.action_ok url=["+item["url"]+"]")
            url = item["url"]
            plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.action_ok url=["+url+"]")
            if "|" in url:
                if plugintools.get_setting("quality_selector")=="ask":
                    self.show_dialog()

                elif plugintools.get_setting("quality_selector")=="sd":
                    url = item["url"].split("|")[0]
                    self.session.open(CustomPlayer, [{"title":item["title"],"url":url,"thumbnail":item["thumbnail"],"plot":item["plot"]}] , 0 )

                elif plugintools.get_setting("quality_selector")=="hd":
                    url = item["url"].split("|")[1]
                    self.session.open(CustomPlayer, [{"title":item["title"],"url":url,"thumbnail":item["thumbnail"],"plot":item["plot"]}] , 0 )

            else:
                self.session.open(CustomPlayer, [{"title":item["title"],"url":url,"thumbnail":item["thumbnail"],"plot":item["plot"]}] , 0 )

        elif item["action"]=="back":
            self.action_back()

        elif item["title"]=="(Empty list)":
            self.action_back()

        elif item["action"]=="tvshows_seasons":
            self.show_loader()
            tvshow = item["title"]
            api.get_tvshow_seasons(plugintools.get_setting("token") , tvshow , num_page = "0", per_page="12", callback = self.callback_after_api, action=item["action"], title="RUYA / TV SHOWS / "+tvshow.upper())

        elif item["action"] in ["movies_recent","latest_releases","movies_by_letter","movies_by_genre","tvshows_updated","tvshows_by_letter","tvshows_by_genre","movies_search_results","tvshows_search_results"]:
            self.show_loader()
            if "num_page" in item:
                num_page = item["num_page"]
            else:
                num_page = "0"

            if "per_page" in item:
                per_page = item["per_page"]
            else:
                per_page = "10"

            if item["action"]=="movies_recent":
                api.get_recent_movies(plugintools.get_setting("token") , num_page = num_page, per_page = per_page, callback = self.callback_after_api, action=item["action"], title=self.screen_title)
            elif item["action"]=="latest_releases":
                api.get_latest_releases(plugintools.get_setting("token") , num_page = num_page, per_page = per_page, callback = self.callback_after_api, action=item["action"], title=self.screen_title)
            elif item["action"]=="movies_by_letter":
                letter = item["extra"]
                api.get_movies_by_letter(plugintools.get_setting("token") , letter , num_page = num_page, per_page = per_page, callback = self.callback_after_api, action=item["action"], title=self.screen_title)
            elif item["action"]=="movies_by_genre":
                genre = item["extra"]
                api.get_movies_by_genre(plugintools.get_setting("token") , genre , num_page = num_page, per_page = per_page, callback = self.callback_after_api, action=item["action"], title=self.screen_title)
            elif item["action"]=="movies_search_results":
                self.hide_loader()
                terms = item["extra"]
                new_itemlist = api.movie_search(plugintools.get_setting("token") , terms , num_page = num_page, per_page = per_page, callback = self.callback_after_api, action=item["action"], title=self.screen_title)
                self.session.open(RuyaBrowseMoviesScreen,new_itemlist,self.screen_title)
            elif item["action"]=="tvshows_updated":
                api.get_updated_tvshows(plugintools.get_setting("token") , num_page = num_page, per_page = per_page, callback = self.callback_after_api, action=item["action"], title=self.screen_title)
            elif item["action"]=="tvshows_by_letter":
                letter = item["extra"]
                api.get_tvshows_by_letter(plugintools.get_setting("token") , letter , num_page = num_page, per_page = per_page, callback = self.callback_after_api, action=item["action"], title=self.screen_title)
            elif item["action"]=="tvshows_by_genre":
                genre = item["extra"]
                api.get_tvshows_by_genre(plugintools.get_setting("token") , genre , num_page = num_page, per_page = per_page, callback = self.callback_after_api, action=item["action"], title=self.screen_title)
            elif item["action"]=="tvshows_search_results":
                self.hide_loader()
                terms = item["extra"]
                new_itemlist = api.tvshow_search(plugintools.get_setting("token") , terms , num_page = num_page, per_page = per_page)
                self.session.open(RuyaBrowseMoviesScreen,new_itemlist,self.screen_title)

    def callback_after_api(self,action,title,items):
        plugintools.loginfo("ruya.RuyaSectionMoviesScreen.callback_after_api")

        self.hide_loader()

        try:
            if action in ["tvshows_seasons"]:
                from screen_section_tvshows import RuyaSectionTVShowsScreen
                self.session.open(RuyaSectionTVShowsScreen,items,title)
            else:
                self.session.open(RuyaBrowseMoviesScreen,items,title)
        except:
            import traceback
            plugintools.loginfo(traceback.format_exc())

    def action_back(self):
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.action_back")

        if self.dialog_opened:
            self.hide_dialog()
        else:
            self.close(False,self.session)

    def scroll_up(self):

        if (self.first_item_on_screen+self.NUM_ITEMS_ON_SCREEN) < len(self.itemlist):
            self.first_item_on_screen = self.first_item_on_screen + 1
            self.repaint_list()

            # Updates metadata for episode
            item = self.itemlist[self.first_item_on_screen + self.active]

            self["title"].setText( item["title"] )
            self["plot"].setText( item["plot"] )

            self.thumbnail = item["thumbnail"]
            self.load_thumbnail()

    def scroll_down(self):

        if self.first_item_on_screen>0:
            self.first_item_on_screen = self.first_item_on_screen - 1
            self.repaint_list()

            # Updates metadata for episode
            item = self.itemlist[self.first_item_on_screen + self.active]

            self["title"].setText( item["title"] )
            self["plot"].setText( item["plot"] )

            self.thumbnail = item["thumbnail"]
            self.load_thumbnail()

    def repaint_list(self):
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.repaint_list")

        for index_on_screen in range(0,self.NUM_ITEMS_ON_SCREEN):
            index_on_itemlist = index_on_screen + self.first_item_on_screen

            if index_on_itemlist < len(self.itemlist):
                self["title_option"+str(index_on_screen)].setText( self.itemlist[index_on_itemlist]["title"] )
            else:
                self["title_option"+str(index_on_screen)].setText( "" )

        self.update_scrollbar_pointer()

    def update_scrollbar_pointer(self):
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.update_scrollbar_pointer")

        if self["scrollbar_section"].visible:
            # Get scrollbar coordinates and size
            scrollbar_x , scrollbar_y = self["scrollbar_section"].getPosition()
            plugintools.loginfo("scrollbar_x="+str(scrollbar_x)+", scrollbar_y="+str(scrollbar_y))

            scrollbar_height = 539

            # Get current item on itemlist
            index_on_itemlist = self.active + self.first_item_on_screen
            plugintools.loginfo("index_on_itemlist="+str(index_on_itemlist))

            # Calculates pointer coordinates
            step_y = scrollbar_height // len(self.itemlist)
            plugintools.loginfo("step_y="+str(step_y))

            pointer_x = scrollbar_x
            pointer_y = scrollbar_y + (step_y * index_on_itemlist)
            plugintools.loginfo("pointer_x="+str(pointer_x)+", pointer_y="+str(pointer_y))

            self["scrollbar_pointer"].move(pointer_x,pointer_y)

    def action_up(self):
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.action_up active=["+str(self.active)+"]")

        if self.dialog_opened:
            self["quality_selector_sd_focus"].show()
            self["quality_selector_sd_text"].setForegroundColorNum(1)
            self["quality_selector_hd_focus"].hide()
            self["quality_selector_hd_text"].setForegroundColorNum(0)
            self.dialog_selected = "sd"
        else:

            if self.active>0:
                self.active = self.active - 1
                self.move_focus()
            else:
                if len(self.itemlist)>self.NUM_ITEMS_ON_SCREEN:
                    self.scroll_down()
                else:
                    self.active = self.MAX_ITEM
                    self.move_focus()

            self.update_scrollbar_pointer()

            # Updates metadata for episode
            item = self.itemlist[self.first_item_on_screen + self.active]

            self["title"].setText( item["title"] )
            self["plot"].setText( item["plot"] )

            self.thumbnail = item["thumbnail"]

            pixmap = LoadPixmap("/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/aeon/movie-thumbnail-black.png")
            self["thumbnail"].instance.setPixmap(pixmap)

            self.load_thumbnail()

    def action_down(self):
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.action_down active=["+str(self.active)+"]")

        if self.dialog_opened:
            self["quality_selector_sd_focus"].hide()
            self["quality_selector_sd_text"].setForegroundColorNum(0)
            self["quality_selector_hd_focus"].show()
            self["quality_selector_hd_text"].setForegroundColorNum(1)
            self.dialog_selected = "hd"
        else:
            if self.active<self.MAX_ITEM:
                self.active = self.active + 1
                self.move_focus()
            else:
                if len(self.itemlist)>self.NUM_ITEMS_ON_SCREEN:
                    self.scroll_up()
                else:
                    self.active = 0
                    self.move_focus()

            self.update_scrollbar_pointer()

            # Updates metadata for episode
            item = self.itemlist[self.first_item_on_screen + self.active]

            self["title"].setText( item["title"] )
            self["plot"].setText( item["plot"] )

            self.thumbnail = item["thumbnail"]

            pixmap = LoadPixmap("/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/aeon/movie-thumbnail-black.png")
            self["thumbnail"].instance.setPixmap(pixmap)

            self.load_thumbnail()

    def action_left(self):
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.action_left first_item_on_screen="+str(self.first_item_on_screen)+", active="+str(self.active))

        self.first_item_on_screen = self.first_item_on_screen - self.NUM_ITEMS_ON_SCREEN
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.action_left first_item_on_screen="+str(self.first_item_on_screen)+", active="+str(self.active))

        if self.first_item_on_screen<0:
            self.first_item_on_screen = 0
            self.active = 0
            plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.action_left first_item_on_screen="+str(self.first_item_on_screen)+", active="+str(self.active))

        self.repaint_list()
        self.move_focus()

        # Updates metadata for episode
        item = self.itemlist[self.first_item_on_screen + self.active]

        self["title"].setText( item["title"] )
        self["plot"].setText( item["plot"] )

        self.thumbnail = item["thumbnail"]

        pixmap = LoadPixmap("/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/aeon/movie-thumbnail-black.png")
        self["thumbnail"].instance.setPixmap(pixmap)

        self.load_thumbnail()

    def action_right(self):
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.action_right first_item_on_screen="+str(self.first_item_on_screen)+", active="+str(self.active))

        self.first_item_on_screen = self.first_item_on_screen + self.NUM_ITEMS_ON_SCREEN
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.action_right first_item_on_screen="+str(self.first_item_on_screen)+", active="+str(self.active))

        if (self.first_item_on_screen + self.NUM_ITEMS_ON_SCREEN) > len(self.itemlist):

            self.first_item_on_screen = len(self.itemlist) - self.NUM_ITEMS_ON_SCREEN
            if self.first_item_on_screen < 0:
                self.first_item_on_screen = 0

            self.active = self.NUM_ITEMS_ON_SCREEN-1
            if self.active>len(self.itemlist)-1:
                self.active = len(self.itemlist)-1

            plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.action_right first_item_on_screen="+str(self.first_item_on_screen)+", active="+str(self.active))

        self.repaint_list()
        self.move_focus()

        # Updates metadata for episode
        item = self.itemlist[self.first_item_on_screen + self.active]

        self["title"].setText( item["title"] )
        self["plot"].setText( item["plot"] )

        self.thumbnail = item["thumbnail"]

        pixmap = LoadPixmap("/usr/lib/enigma2/python/Plugins/Extensions/ruya/resources/img/aeon/movie-thumbnail-black.png")
        self["thumbnail"].instance.setPixmap(pixmap)

        self.load_thumbnail()

    def move_focus(self):
        plugintools.loginfo("ruya.RuyaBrowseMoviesScreen.move_focus active=["+str(self.active)+"]")

        #active_x,active_y = self["background_option"+str(self.active)].getPosition()
        #self["focus"].move(active_x-6,active_y-6)
        for i in range(0,self.NUM_ITEMS_ON_SCREEN):
            if i==self.active:
                self["focus"+str(i)].show()
                self["title_option"+str(i)].setForegroundColorNum(1)
            else:
                self["focus"+str(i)].hide()
                self["title_option"+str(i)].setForegroundColorNum(0)

    def show_dialog(self):
        self.dialog_opened = True
        self["quality_selector_background"].show()
        self["quality_selector_sd_focus"].show()
        self["quality_selector_hd_focus"].hide()
        self["quality_selector_sd_text"].show()
        self["quality_selector_hd_text"].show()
        self["quality_selector_sd_text"].setForegroundColorNum(1)
        self["quality_selector_hd_text"].setForegroundColorNum(0)
        self.dialog_selected="sd"

    def hide_dialog(self):
        self.dialog_opened = False
        self["quality_selector_background"].hide()
        self["quality_selector_sd_focus"].hide()
        self["quality_selector_hd_focus"].hide()
        self["quality_selector_sd_text"].hide()
        self["quality_selector_hd_text"].hide()
        self.dialog_selected="sd"
