# -*- coding: utf-8 -*-
# Code and idea taken from Mediaportal plugin

import plugintools

from Screens.MessageBox import MessageBox
from Screens.Standby import TryQuitMainloop

from enigma import eConsoleAppContainer

from twisted.web.client import getPage

class Updater:
    
    def __init__(self, session):
        self.session = session

    def check_for_updates(self):
        
        try:
            update_url="http://"+plugintools.get_setting("server")+"/ruyaserver/version.txt"
            plugintools.loginfo("Updater.check_for_updates "+update_url)
            getPage(update_url).addCallback(self.got_update_info).addErrback(self.got_error)
        except:
            import traceback
            plugintools.loginfo(traceback.format_exc())

    def got_error(self, error=""):
        plugintools.loginfo("Updater.got_error error="+repr(error))
        return

    def got_update_info(self, html):
        plugintools.loginfo("Updater.got_update_info html="+repr(html))

        plugintools.loginfo("Updater.got_update_info local version="+repr(plugintools.get_setting("version")))

        tmp_infolines = html.splitlines()
        remoteversion = tmp_infolines[0]
        plugintools.loginfo("Updater.got_update_info remoteversion="+repr(remoteversion))

        self.updateurl = tmp_infolines[1]
        plugintools.loginfo("Updater.got_update_info updateurl="+repr(self.updateurl))

        try:
            if int(plugintools.get_setting("version")) < int(remoteversion):
                plugintools.loginfo("Updater.got_update_info Ask for update")
                self.session.openWithCallback(self.start_plugin_update,MessageBox,_("An update is available for this plugin!\nDo you want to download and install it now?"), MessageBox.TYPE_YESNO)
            else:
                return
        except:
            import traceback
            plugintools.loginfo(traceback.format_exc())
            return

    def start_plugin_update(self, answer):
        plugintools.loginfo("Updater.start_plugin_update")
        if answer is True:
            self.container=eConsoleAppContainer()
            self.container.appClosed.append(self.finished_plugin_update)
            comando = "opkg install --force-overwrite --force-depends " + str(self.updateurl)
            plugintools.loginfo("Updater.start_plugin_update comando="+comando)
            self.container.execute(comando)
        else:
            return

    def finished_plugin_update(self,retval):
        self.session.openWithCallback(self.restart_gui, MessageBox, _("Plugin successfully updated!\nDo you want to restart the system now?"), MessageBox.TYPE_YESNO)

    def restart_gui(self, answer):
        if answer is True:
            self.session.open(TryQuitMainloop, 3)
        else:
            return
